#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-
## @pa10api
#
# Robot Arm(MHI-PA10) Resolved Motion Rate Control Components.

## @Library Source Path.
# To Be Editted by User.

# importするモジュールのpath
import sys
import time
sys.path.append("../../etc")
from set_env import *
sys.path.append(LibDir)
from rtc_handle import *
sys.path.append(SrcDir+'/tools/geo')
from geo import *
sys.path.append(BinDir+"/comp")
import _GlobalIDL

##
#  handle開始
def assign_variables(env) :
  # rtc
  global pa10vel, RMRC, ACT_PA10_RH
  RMRC = env.name_space[RMRC_NS].rtc_handles['RMRC0.rtc']
  pa10vel = env.name_space[DEVICE_NS].rtc_handles['pa10vel0.rtc']
  ACT_PA10_RH = env.name_space[ RMRC_NS ].rtc_handles['ACT_PA10_RH0.rtc']
  ACT_PA10_RH.services['MiddleProv'].provided['ManipulatorCommonInterface_Middle'].narrow_ref(globals())
  ACT_PA10_RH.services['ACTProv'].provided['com_ACT'].narrow_ref(globals())
  RMRC.services['RMRCProv'].provided['com_fk'].narrow_ref(globals())
  RMRC.services['RMRCProv'].provided['com_move'].narrow_ref(globals())
  RMRC.conf_ref.activate_configuration_set('PA10')
  # data & class
  global can1, can2, can3, abv_can1, abv_can2, abv_can3
  can1 = FRAME(xyzabc=[ 635, 316, 356 + 40, 0, 180.0, 0])
  can2 = FRAME(xyzabc=[ 526, 431, 355 + 40, 0, 180.0, 0])
  can3 = FRAME(xyzabc=[ 418, 542, 356 + 40, 0, 180.0, 0])
  abv_can1 = FRAME(xyzabc=[ 635, 316, 356 + 200, 180.0, 0, 0])
  abv_can2 = FRAME(xyzabc=[ 526, 431, 355 + 200, 180.0, 0, 0])
  abv_can3 = FRAME(xyzabc=[ 418, 542, 356 + 200, 180.0, 0, 0])

  global c_frame, m_tool_frm, c_xyz, c_abc, c_dseq, m_xyz, m_abc, m_dseq, c_vel, m_vel
  c_frame=_GlobalIDL.Frame
  c_xyz = _GlobalIDL.Vector
  c_abc = _GlobalIDL.Vector
  c_dseq =_GlobalIDL.DoubleSeq
  c_vel =_GlobalIDL.DoubleSeq
  m_xyz = c_xyz(VECTOR())
  m_abc = c_abc(VECTOR())
  m_dseq = c_dseq([0.0,0.0,0.0,0.0,0.0,0.0,0.0])
  m_vel = c_vel([0.0,0.0,0.0,0.0,0.0,0.0,0.0])
  m_tool_frm = c_frame(MATRIX(),VECTOR())

def make_connectors(env):
  env.con_list=[]
#0
  con = IOConnector([pa10vel.outports['th'],RMRC.inports['Th']])
  env.con_list.append(con)
#1
  con = IOConnector([RMRC.outports['Vel'],pa10vel.inports['vel']])
  env.con_list.append(con)
#2
  con = ServiceConnector([ACT_PA10_RH.services['RMRCCons'],RMRC.services['RMRCProv']])
  env.con_list.append(con)
#
# ポート接続
#
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()

##
#
def mode_rmrc(env) :
  deactivate()
  time.sleep(2)
  connect_list(env,[0,1,2])
  activate_seq([pa10vel, RMRC, ACT_PA10_RH, 1])
  set_tool([0,0,150.0])

def activate_seq(a_seq) :
  for tmp in a_seq :
    if isinstance(tmp,RtcHandle) :
      tmp.activate()
    elif isinstance(tmp, int) :
      time.sleep(tmp)
    elif isinstance(tmp, float) :
      time.sleep(tmp)
    else :
      eval(tmp)

def deactivate() :
  pa10vel.deactivate()
  ACT_PA10_RH.deactivate()
  RMRC.deactivate()

def end(env) :
  deactivate()
  disconnect_all(env)
##
# 各軸の関節 [degree]
#
j_park=[0.0,0.0,0.0,0.0,0.0,0.0,0.0]
j_ready=[0.0,20.0,0.0,60.2,0.0,100.3,0.0]
j_holder=[-45.0,20.0,0.0,60.2,0.0,100.3,0.0]
j_table=[45.0,20.0,0.0,60.2,0.0,100.3,0.0]
##
# 角速度制御
#   jlist:各軸の関節 List型 [x,y,z,a,b,c](mm,degree)
def move_joint(jlist) :
   global m_dseq
   for i in range(7):
     m_dseq.data[i] = jlist[i]*pi/180.0
   RMRC.services['RMRCProv'].provided['com_move'].ref.move_joint(m_dseq)
##
# サンプル移動目標値
#
pa     = [600, 0.0, 220, 0.0, 180.0, 180.0/4.0]
pb     = [600, 0.0, 500, 0.0, 180.0, 180.0/4.0]
pc     = [600, 300, 300, 0.0, 180.0, 180.0/4.0]
point1 = [600, 0, 500, 0.0, 180.0, 180.0/4.0]
##
# 手先の現在位置・姿勢を取得
def get_pos():
   cur_pos = RMRC.services['RMRCProv'].provided['com_fk'].ref.get_cur_frm()
   print"current position:"
   print" ROT MATRIX"
   for i in range(3):
     for j in range(3):
       print"  %.3f  " %cur_pos.mat[i][j],  
     print""
   print" POSITION"
   for i in range(3):
     print"  %.3f  " %cur_pos.pos[i],  
   print""
   
##
# 手先ツール長設定
#   tool:ツール(把持点)オフセット List型 [x,y,z](mm)
def set_tool(tool):
   global m_xyz, m_abc
   for i in range(3):
     m_xyz.pos[i] = tool[i]
   for i in range(3):
     m_abc.pos[i] = 0.0
   tl = ACT_PA10_RH.services['ACTProv'].provided['com_ACT'].ref.createCarPosWithElbow(m_xyz,m_abc)
   ACT_PA10_RH.services['MiddleProv'].provided['ManipulatorCommonInterface_Middle'].ref.setControlPointOffset(tl.carPos)

##
# 角速度制御
#    pos:移動目標値 List型 [x,y,z,a,b,c](mm,degree)
def go_to(pos) :
   global m_xyz, m_abc
   for i in range(3):
     m_xyz.pos[i] = pos[i]
     m_abc.pos[i] = pos[i+3]
   m_pos = ACT_PA10_RH.services['ACTProv'].provided['com_ACT'].ref.createCarPosWithElbow(m_xyz,m_abc)
   ACT_PA10_RH.services['MiddleProv'].provided['ManipulatorCommonInterface_Middle'].ref.moveLinearCartesianAbs(m_pos)
##
# 連続移動（角速度制御）
def demo(pos_list) :
     for pos in pos_list :
       go_to(pos)
##
# 移動先確認（可動範囲内か否かのチェック） (可動半径:850mm, LB(設置面から2軸までのリンク長) = 317mm)
#    pos:移動目標値 List型 [x,y,z,a,b,c](mm,degree)
def can_move(pos) :
  if(pos[0]*pos[0] + pos[1]*pos[1] + (pos[2] - 317)*(pos[2] - 317) >= 850*850) :
    # ERROR: 可動範囲外
    print"\n  Unreachable TARGET! \n"
    return 1
  else :
    # 可動範囲内
    return 0
##
# 最大関節動作速度の設定
def set_joint_vel(vel):
  global m_vel
  for i in range(7):
    m_vel.data[i] = vel
  RMRC.services['RMRCProv'].provided['com_move'].ref.set_max_joint_vel(m_vel)
##
#
def start_handle(env) :
  assign_variables(env)
  make_connectors(env)


###############################
#  start a demonstration      #
###############################
env = RtmEnv(sys.argv,[ RMRC_NS, DEVICE_NS ])
for ns in env.name_space :
  env.name_space[ns].list_obj()
start_handle(env)
mode_rmrc(env)
move_joint(j_ready)

