// -*- C++ -*-
/*!
 * @file  PA10SystemController.h
 * @brief PA10システム制御
 * @date  $Date$
 *
 * $Id$
 */
#include <string>
#include <pthread.h>
#ifndef PA10SYSTEMCONTROLLER_H
#define PA10SYSTEMCONTROLLER_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "PA10SystemControllerSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "ControlTerminalStub.h"
#include "com_RMRCStub.h"
#include "com_handStub.h"
#include "VisionStub.h"

// </rtc-template>

void RTCActiveCheck(const char* args);
float calc_tm_dif(RTC::Time tm1, RTC::Time tm2);

using namespace RTC;

/*!
 * @class PA10SystemController
 * @brief PA10システム制御
 *
 */
class PA10SystemController
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  PA10SystemController(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~PA10SystemController();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>
  std::string m_scenario;
  std::string m_check;
  
  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_recogRH_data;
  /*!
   */
  InPort<TimedDoubleSeq> m_recogRH_dataIn;
  TimedDoubleSeq m_recogDrink_data;
  /*!
   */
  InPort<TimedDoubleSeq> m_recogDrink_dataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_PA10SysCtrlrProvPort;
  /*!
   */
  RTC::CorbaPort m_ControlTerminalConsPort;
  /*!
   */
  RTC::CorbaPort m_ComRMRCConsPort;
  /*!
   */
  RTC::CorbaPort m_ComHandConsPort;
  /*!
   */
  RTC::CorbaPort m_RecogRHConsPort;
  /*!
   */
  RTC::CorbaPort m_RecogDrinkConsPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   */
  PA10SystemControlSVC_impl m_system_control;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<StateChangeFromPA10> m_state_change_from_pa10;
  /*!
   */
  RTC::CorbaConsumer<ComFrmCtrl> m_com_frm_ctrl;
  /*!
   */
  RTC::CorbaConsumer<ComFk> m_com_fk;
  /*!
   */
  RTC::CorbaConsumer<ComMove> m_com_move;
  /*!
   */
  RTC::CorbaConsumer<ComHand> m_com_hand;
  /*!
   */
  RTC::CorbaConsumer<RecognitionService> m_recogRH;
  /*!
   */
  RTC::CorbaConsumer<RecognitionService> m_recogDrink;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
  RTC::Time start_tm, old_tm;
  int red_flag;
  int sv_flag;
};


extern "C"
{
  DLL_EXPORT void PA10SystemControllerInit(RTC::Manager* manager);
};

#endif // PA10SYSTEMCONTROLLER_H
