// -*-C++-*-
/*!
 * @file  ManipulatorCommonInterface_MiddleLevelSVC_impl.cpp
 * @brief Service implementation code of ManipulatorCommonInterface_MiddleLevel.idl
 *
 */

#include "ManipulatorCommonInterface_MiddleLevelSVC_impl.h"
#include <iostream>

/*
 * Example implementational code for IDL interface ManipulatorCommonInterface_Middle
 */
ManipulatorCommonInterface_MiddleSVC_impl::ManipulatorCommonInterface_MiddleSVC_impl()
:rtclog("ManipulatorCommonInterface_MiddleSVC_impl")
{
  // Please add extra constructor code here.
}


ManipulatorCommonInterface_MiddleSVC_impl::~ManipulatorCommonInterface_MiddleSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * グリッパを閉じる
 */
RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::closeGripper()
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  if (::CORBA::is_nil((adrs_com_hand_cons)->getObject()) == false){
     ID->id = (*adrs_com_hand_cons)->CloseHand(0);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("CloseGripper() was completed."));
  return ID;
}

/*
 * マニピュレータの設置位置を取得
 */
RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getBaseOffset(RTC::HgMatrix offset)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();

  RTC_INFO(("getBaseOffset() was completed.\n"
             "      offset :[%f, %f, %f, %f \n"
             "                 %f, %f, %f, %f \n"
             "                 %f, %f, %f, %f] \n"
             ,offset[0][0],offset[0][1],offset[0][2],offset[0][3]
             ,offset[1][0],offset[1][1],offset[1][2],offset[1][3]
             ,offset[2][0],offset[2][1],offset[2][2],offset[2][3]));

  ID->id = 0;
  ID->comment = "OK";
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getFeedbackPosCartesian(RTC::CarPosWithElbow& pos)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  int i,j;
  Frame out;
  
  if (::CORBA::is_nil((adrs_com_fk_cons)->getObject()) == false){
     //get_cur_frm()の呼び出し
     out = (*adrs_com_fk_cons)->get_cur_frm();   
     //Frame型 cur_frm を CarPosWithElbow型 pos に変換する
     for(i=0;i<3;i++){
       for(j=0;j<3;j++){
         pos.carPos[i][j] = out.mat[i][j];
        }
       pos.carPos[i][3] = out.pos[i];
      } 
     pos.elbow = 0.0;    
     pos.structFlag = 0;
     RTC_INFO(("getFeedbackPosCartesian() was completed.\n"
                "  pos.carPos :[%f, %f, %f, %f \n"
                "                 %f, %f, %f, %f \n"
                "                 %f, %f, %f, %f] \n"
                ,pos.carPos[0][0],pos.carPos[0][1],pos.carPos[0][2],pos.carPos[0][3]
                ,pos.carPos[1][0],pos.carPos[1][1],pos.carPos[1][2],pos.carPos[1][3]
                ,pos.carPos[2][0],pos.carPos[2][1],pos.carPos[2][2],pos.carPos[2][3]));

     ID->id = 0;
     ID->comment = "OK";
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getMaxSpeedCartesian(RTC::CartesianSpeed& speed)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getMaxSpeedJoint(RTC::DoubleSeq_out speed)
{

  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
   
  ID->id = 0;
  ID->comment = "OK";
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getMinAccelTimeCartesian(::CORBA::Double& aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getMinAccelTimeJoint(::CORBA::Double& aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::getSoftLimitCartesian(RTC::LimitValue& xLimit, RTC::LimitValue& yLimit, RTC::LimitValue& zLimit)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  if (::CORBA::is_nil((adrs_com_move_cons)->getObject()) == false){
     ID->id = (*adrs_com_move_cons)->get_softlimit(xLimit.upper, yLimit.upper, zLimit.upper, xLimit.lower, yLimit.lower, zLimit.lower);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }

  RTC_INFO(("getSoftLimitCartesian() was completed."));
  RTC_INFO(("Max softlimit on X axis = %f", xLimit.upper));
  RTC_INFO(("Max softlimit on Y axis = %f", yLimit.upper));
  RTC_INFO(("Max softlimit on Z axis = %f", zLimit.upper));
  RTC_INFO(("Min softlimit on X axis = %f", xLimit.lower));
  RTC_INFO(("Min softlimit on Y axis = %f", yLimit.lower));
  RTC_INFO(("Min softlimit on Z axis = %f", zLimit.lower));
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::moveGripper(RTC::ULONG angleRatio)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::moveLinearCartesianAbs(const RTC::CarPosWithElbow& carPoint)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  int i,j;
  Frame frm;
    
  for(i=0; i<3; i++) {
    for(j=0; j<3; j++) {
      frm.mat[i][j] = carPoint.carPos[i][j];
     }
    frm.pos[i] = carPoint.carPos[i][3];
  }
  
  if (::CORBA::is_nil((adrs_com_move_cons)->getObject()) == false){
     ID->id = (*adrs_com_move_cons)->move_to(frm);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("moveLinearCartesianAbs() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::moveLinearCartesianRel(const RTC::CarPosWithElbow& carPoint)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::movePTPCartesianAbs(const RTC::CarPosWithElbow& carPoint)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::movePTPCartesianRel(const RTC::CarPosWithElbow& carPoint)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::movePTPJointAbs(const RTC::JointPos& jointPoints)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  int i;
  int size;
  DoubleSeq points;
  
  size = jointPoints.length();
  points.data.length(size);
  for(i=0;i<size;i++){
    points.data[i] = jointPoints[i] * PI/180.0;
  }
  if (::CORBA::is_nil((adrs_com_move_cons)->getObject()) == false){
     ID->id = (*adrs_com_move_cons)->move_joint(points);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("movePTPJointAbs() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::movePTPJointRel(const RTC::JointPos& jointPoints)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}
/*
 * グリッパを開く
 */
RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::openGripper()
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  if (::CORBA::is_nil((adrs_com_hand_cons)->getObject()) == false){
     ID->id = (*adrs_com_hand_cons)->OpenHand(0);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("openGripper() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::pause()
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::resume()
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::stop()
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setAccelTimeCartesian(::CORBA::Double aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setAccelTimeJoint(::CORBA::Double aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setBaseOffset(const RTC::HgMatrix offset)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setControlPointOffset(const RTC::HgMatrix offset)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  int i,j;
  Frame tool_frm;
    
  for(i=0; i<3; i++) {
    for(j=0; j<3; j++) {
      tool_frm.mat[i][j] = offset[i][j];
     }
    tool_frm.pos[i] = offset[i][3];
  }
  
  if (::CORBA::is_nil((adrs_com_fk_cons)->getObject()) == false){
     ID->id = (*adrs_com_fk_cons)->set_tool(tool_frm);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("setControlPointOffset() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setMaxSpeedCartesian(const RTC::CartesianSpeed& speed)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setMaxSpeedJoint(const RTC::DoubleSeq& speed)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  int i;
  DoubleSeq spd;
  
  j_num = speed.length();
  spd.data.length(j_num);
  max_speed_joint.length(j_num);
  
  for(i=0;i<j_num;i++){
    max_speed_joint[i] = speed[i] * PI/180.0;
    spd.data[i] = speed[i] * PI/180.0;
  }
  if (::CORBA::is_nil((adrs_com_move_cons)->getObject()) == false){
     ID->id = (*adrs_com_move_cons)->set_max_joint_vel(spd);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("setMaxSpeedJoint() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setMinAccelTimeCartesian(::CORBA::Double aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setMinAccelTimeJoint(::CORBA::Double aclTime)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setSoftLimitCartesian(const RTC::LimitValue& xLimit, const RTC::LimitValue& yLimit, const RTC::LimitValue& zLimit)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  if (::CORBA::is_nil((adrs_com_move_cons)->getObject()) == false){
     ID->id = (*adrs_com_move_cons)->set_softlimit(xLimit.upper, yLimit.upper, zLimit.upper, xLimit.lower, yLimit.lower, zLimit.lower);
     if(ID->id == 0){
       ID->comment = "OK";
     }else {
      	ID->id = -1;
       ID->comment = "NG";
     }
  }else {
    	RTC_INFO(("port connect error"));
    	ID->id = -1;
     ID->comment = "NG";
  }
  RTC_INFO(("setSoftLimitCartesian() was completed."));
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setSpeedCartesian(RTC::ULONG spdRatio)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}

RTC::RETURN_ID* ManipulatorCommonInterface_MiddleSVC_impl::setSpeedJoint(RTC::ULONG spdRatio)
{
  RTC::RETURN_ID* ID = new RTC::RETURN_ID();
  
  std::cout << "Not supported." << std::endl;
  ID->id = 0;
  ID->comment = "OK";
  
  return ID;
}



// End of example implementational code



