// -*-C++-*-
/*!
 * @file  com_handSVC_impl.cpp
 * @brief Service implementation code of com_hand.idl
 *
 */

#include "com_handSVC_impl.h"

/*
 * Example implementational code for IDL interface ComHand
 */
ComHandSVC_impl::ComHandSVC_impl()
:rtclog("ComHandSVC_impl")
{
  hDrv = INVALID_HANDLE_VALUE;
}


ComHandSVC_impl::~ComHandSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::Short ComHandSVC_impl::InitHand()
{
  int err;

  err = DioOpen(&hDrv, 2, 1);			// Open process
  if(err != 0) {
	  RTC_ERROR(("DioOpen error!"));
	  return (short)err;
  }
  if((err = OpenHand(0)) != 0){						// initial hand open
	  return (short)err;
  }

  this->InpPortNo[0] = 1;							// Set port 0 to InpPortNo[0]
  this->lpDInp.InpPortNo = this->InpPortNo;			// Set the pointer of InpPortNo
  this->lpDInp.PortNum = 1;							// Total numbe of logical ports
  this->lpDInp.Buf = this->BufIn;					// Set the pointer of Buf
  do{
	  err = DioInp(this->hDrv,&this->lpDInp);
	  if(err != 0){
		  RTC_ERROR(("DioInp error!: %x\n",err));
		  return (short)err;
	  }
  }while(this->lpDInp.Buf[0] != 0x2);

  this->cond = 2;
  this->old_bit = 0x2;						// Hand opened

  err = DioClose(hDrv);						// Close process
  if(err != 0) {
	  RTC_ERROR(("DioClose error!"));
	  return (short)err;
  }

  RTC_ERROR(("Successfuly Initialized.\n"));

  return (short)err;
}

CORBA::Short ComHandSVC_impl::OpenHand(CORBA::Short slow)
{
  int err;
  
  this->cond = 1;							// set as "now moving"
  
  OutPortNo[0] = 1;							// Set port 0 to InpPortNo[0]
  LpDOut.OutPortNo = OutPortNo;				// Set the pointer of OutPortNo
  LpDOut.PortNum = 1;						// Total numbe of logical ports
  if(slow == 0){
	  this->mode = 1;						// normal speed
	  BufOut[0] = 0x0;						// Output data to port 0
	  LpDOut.Buf = BufOut;					// Set the pointer of Buf
	  RTC_DEBUG(("Start OPEN ** normal **"));
	  err = DioOut(hDrv, &LpDOut);			// hand open
	  if(err != 0){
		  RTC_ERROR(("hand open error!"));
		  return (short)err;
	  }
  }else if(slow == 1){
	  this->mode = 2;						// slow speed
	  BufOut[0] = 0x2;						// Output data to port 0
	  LpDOut.Buf = BufOut;					// Set the pointer of Buf
	  RTC_DEBUG(("Start OPEN ** slow **"));
	  err = DioOut(hDrv, &LpDOut);			// hand open slowly
	  if(err != 0){
		  RTC_ERROR(("hand open error!"));
		  return (short)err;
	  }
  }else {
	  RTC_ERROR(("Invalid parameter for OpenHand()"));
  }

  return (short)err;
}

CORBA::Short ComHandSVC_impl::CloseHand(CORBA::Short slow)
{
  int err;
  
  this->cond = 1;							//now moving

  OutPortNo[0] = 1;							// Set port 0 to InpPortNo[0]
  LpDOut.OutPortNo = OutPortNo;				// Set the pointer of OutPortNo
  LpDOut.PortNum = 1;						// Total numbe of logical ports
  if(slow == 0){
	  this->mode = 3;						// normal speed
	  BufOut[0] = 0x1;						// Output data to port 0
	  LpDOut.Buf = BufOut;					// Set the pointer of Buf
	  RTC_DEBUG(("Start CLOSE ** normal **"));
	  err = DioOut(hDrv, &LpDOut);			// hand close
	  if(err != 0){
		  RTC_ERROR(("hand close error!"));
		  return (short)err;
	  }
  }else if(slow == 1){
	  this->mode = 4;						// slow speed
	  BufOut[0] = 0x3;						// Output data to port 0
	  LpDOut.Buf = BufOut;					// Set the pointer of Buf
	  RTC_DEBUG(("Start CLOSE ** slow **"));
	  err = DioOut(hDrv, &LpDOut);			// hand close slowly
	  if(err != 0){
		  RTC_ERROR(("hand close error!"));
		  return (short)err;
	  }
  }else {
	  RTC_ERROR(("Invalid parameter for CloseHand()"));
  }
  
  return (short)err;
}

CORBA::Short ComHandSVC_impl::GetHandCond()
{
  return this->cond;
}

CORBA::Short ComHandSVC_impl::StopHand()
{
  int err;
  
  OutPortNo[0] = 1;						// Set port 0 to InpPortNo[0]
  LpDOut.OutPortNo = OutPortNo;			// Set the pointer of OutPortNo
  LpDOut.PortNum = 1;					// Total numbe of logical ports
  BufOut[0] = 0x4;						// Output data to port 0
  LpDOut.Buf = BufOut;					// Set the pointer of Buf
  err = DioOut(hDrv, &LpDOut);			// interrupt open/close
  if(err != 0){
	  RTC_ERROR(("open/close interrupt error!"));
	  return (short)err;
  }
  RTC_DEBUG(("Opne/Close interrupted."));

  this->cond = 4;						// set as "Interrupted"

  return (short)err;
}

CORBA::Short ComHandSVC_impl::GetHandMode()
{
  return this->mode;
}

// End of example implementational code



